{% extends "!layout.html" %}

{%- block rootrellink %}
        <li><a href="{{ pathto('index') }}">home</a>|&nbsp;</li>
        <li><a href="{{ pathto('contents') }}">contents</a> &raquo;</li>
{%- endblock %}


{%- block relbar1 %}{{ relbar() }}{% endblock %}
{%- block relbar2 %}{% endblock %}

{%- block linktags %}
{{ super() }}
<link rel="top" title="{{ docstitle }}" href="#" />
{%- if '+' in release %}
    <link rel="canonical" href="https://matplotlib.org/devdocs/{{pagename}}.html" />
{%- else %}
    <link rel="canonical" href="https://matplotlib.org/{{version}}/{{pagename}}.html" />
{%- endif %}
{%- endblock %}

{%- block header %}
{{super()}}
{%- if '+' in release %}
<div id="unreleased-message">
    You are reading documentation for the unreleased version of Matplotlib.
    <a href="https://matplotlib.org/search.html?q={{ title | striptags | urlencode }}&amp;check_keywords=yes&amp;area=default">
        Try searching for the released version of this page instead?
    </a>
</div>

{%- endif %}

<!--
<div id="annc-banner">

</div>
-->

<div style="background-color: white; text-align: left; padding: 10px 10px 15px 15px; position: relative;">
    {%- if builder in ('htmlhelp', 'devhelp', 'latex') %}
    <a href="{{ pathto('index') }}">
        <div style="float: left; position: absolute; width: 496px; bottom: 0; padding-bottom: 24px"><span style="float: right; color: #789; background: white">Version {{ version|e }}</span></div>
        <img src="{{pathto("_static/logo2.png", 1) }}" height="125px" border="0" alt="matplotlib"/></a>
    {%- else %}
    <a href="{{ pathto('index') }}">
        <div style="float: left; position: absolute; width: 496px; bottom: 0; padding-bottom: 24px"><span style="float: right; color: #789; background: white">Version {{ version|e }}</span></div>
        <img src="{{pathto("_static/logo2_compressed.svg", 1) }}" height="125px" border="0" alt="matplotlib"/></a>
    {%- endif %}

    <!-- The "Fork me on github" ribbon -->
    <div id="forkongithub"><a href="https://github.com/matplotlib/matplotlib">Fork me on GitHub</a></div>
    </div>

    <nav class="main-nav">
        <ul>
            <li><a href="{{ pathto('users/installing') }}">Installation</a></li>
            <li><a href="{{ pathto('contents') }}">Documentation</a></li>
            <li><a href="{{ pathto('gallery/index') }}">Examples</a></li>
            <li><a href="{{ pathto('tutorials/index') }}">Tutorials</a></li>
            <li><a href="{{ pathto('devel/index') }}">Contributing</a></li>
            <li class="nav-right">
                <form class="search" action="{{ pathto('search') }}" method="get">
                <input type="text" name="q" aria-labelledby="searchlabel" placeholder="Search"/>
                </form>
            </li>
        </ul>
     </nav>
{%- endblock %}

{%- block sidebar1 %}{{ sidebar() }}{% endblock %}
{%- block sidebar2 %}{% endblock %}

{%- block footer %}
<footer>
    <div class="footer">
{%- if show_copyright %}
{%- if hasdoc('copyright') %}
    {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a
href="{{ path }}">Copyright</a> {{ copyright }}.{% endtrans %}
{%- else %}
    {% trans copyright=copyright|e %}&copy; Copyright {{ copyright }}.{%
endtrans %}
{%- endif %}
{%- endif %}
<br />
{%- if last_updated %}
    {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
{%- endif %}
{%- if show_sphinx %}
{% trans sphinx_version=sphinx_version|e %}Created using
<a href="http://sphinx-doc.org/">Sphinx</a> {{ sphinx_version }}.{% endtrans %}
{%- endif %}
{%- if sha %}
Doc version {{ sha }}.
{%- endif %}
    </div>
</footer>
<script>
        (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
        (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
        m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
        })(window,document,'script','//www.google-analytics.com/analytics.js','ga');

        ga('create', 'UA-55954603-1', 'auto');
        ga('send', 'pageview');

</script>
{%- endblock %}
