//----------------------------------------------------------------------------
// Anti-Grain Geometry - Version 2.4
// Copyright (C) 2009 John Horigan (http://www.antigrain.com)
//
// Permission to copy, use, modify, sell and distribute this software 
// is granted provided this copyright notice appears in all copies. 
// This software is provided "as is" without express or implied
// warranty, and with no claim as to its suitability for any purpose.
//
//----------------------------------------------------------------------------
//
// Contact: john@glyphic.com.com
//          http://www.antigrain.com
//----------------------------------------------------------------------------

// rgbaN construction from grayN types is no longer required,
// as grayN types now define their own conversions to rgbaN.
