.. This (-*- rst -*-) format file contains commonly used link targets
   and name substitutions.  It may be included in many files,
   therefore it should only contain link targets and name
   substitutions.  Try grepping for "^\.\. _" to find plausible
   candidates for this list.

.. NOTE: reST targets are
   __not_case_sensitive__, so only one target definition is needed for
   nipy, NIPY, Nipy, etc...

.. git stuff
.. _git: https://git-scm.com/
.. _github: https://github.com
.. _github help: https://help.github.com
.. _msysgit: https://git-scm.com/download/win
.. _git-osx-installer: https://git-scm.com/download/mac
.. _subversion: https://subversion.apache.org/
.. _git cheat sheet: https://help.github.com/git-cheat-sheets/
.. _pro git book: https://git-scm.com/book/en/v2
.. _git svn crash course: https://git-scm.com/course/svn.html
.. _network graph visualizer: https://github.com/blog/39-say-hello-to-the-network-graph-visualizer
.. _git user manual: https://schacon.github.io/git/user-manual.html
.. _git tutorial: https://schacon.github.io/git/gittutorial.html
.. _git community book: https://git-scm.com/book/en/v2
.. _git ready: http://gitready.com/
.. _Fernando's git page: http://www.fperez.org/py4science/git.html
.. _git magic: http://www-cs-students.stanford.edu/~blynn/gitmagic/index.html
.. _git concepts: https://www.sbf5.com/~cduan/technical/git/
.. _git clone: https://schacon.github.io/git/git-clone.html
.. _git checkout: https://schacon.github.io/git/git-checkout.html
.. _git commit: https://schacon.github.io/git/git-commit.html
.. _git push: https://schacon.github.io/git/git-push.html
.. _git pull: https://schacon.github.io/git/git-pull.html
.. _git add: https://schacon.github.io/git/git-add.html
.. _git status: https://schacon.github.io/git/git-status.html
.. _git diff: https://schacon.github.io/git/git-diff.html
.. _git log: https://schacon.github.io/git/git-log.html
.. _git branch: https://schacon.github.io/git/git-branch.html
.. _git remote: https://schacon.github.io/git/git-remote.html
.. _git rebase: https://schacon.github.io/git/git-rebase.html
.. _git config: https://schacon.github.io/git/git-config.html
.. _why the -a flag?: http://gitready.com/beginner/2009/01/18/the-staging-area.html
.. _git staging area: http://gitready.com/beginner/2009/01/18/the-staging-area.html
.. _tangled working copy problem: http://2ndscale.com/rtomayko/2008/the-thing-about-git
.. _git management: https://web.archive.org/web/20090224195437/http://kerneltrap.org/Linux/Git_Management
.. _linux git workflow: https://www.mail-archive.com/dri-devel@lists.sourceforge.net/msg39091.html
.. _git parable: http://tom.preston-werner.com/2009/05/19/the-git-parable.html
.. _git foundation: https://matthew-brett.github.io/pydagogue/foundation.html
.. _deleting master on github: https://matthew-brett.github.io/pydagogue/gh_delete_master.html
.. _rebase without tears: https://matthew-brett.github.io/pydagogue/rebase_without_tears.html
.. _resolving a merge: https://schacon.github.io/git/user-manual.html#resolving-a-merge
.. _ipython git workflow: https://mail.python.org/pipermail/ipython-dev/2010-October/005632.html

.. other stuff
.. _python: https://www.python.org

.. |emdash| unicode:: U+02014

.. vim: ft=rst
